@@ -48,7 +48,7 @@ Now point your browser to the newly created CGI file and start managing your fil
 
 =head1 Version
 
-Version 0.01
+Version 0.02_03
 
 
 =cut
@@ -60,19 +60,20 @@ Version 0.01
 
 =cut
 
-our $VERSION = '0.02_01';
+our $VERSION = '0.02_03';
 
 use base 'CGI::Application';
 use CGI::Application::Session;
 use CGI::Upload;
 use File::Spec;
-use File::Basename;
-use Data::Dumper;
+use File::Basename qw(dirname);
+use Data::Dumper qw(Dumper);
 use HTML::Template;
 #use Fcntl qw(:flock);
 #use POSIX qw(strftime);
-use File::Copy;
+use File::Copy qw(move);
 use Carp qw(cluck croak);
+use CGI::FileManager::Templates;
 
 
 use CGI::FileManager::Auth;
@@ -95,6 +96,13 @@ sub cgiapp_init {
 		SEND_COOKIE         => 1,
 	);
 	
+	if ($self->param("TMPL_PATH")) {
+		$self->tmpl_path([
+			File::Spec->catfile($self->param("TMPL_PATH"), "custom"),
+			File::Spec->catfile($self->param("TMPL_PATH"), "factory"),
+			]);
+	}
+
 	$self->header_props( 
 		-expires => '-1d',  
 		# I think this this -expires causes some strange behaviour in IE 
@@ -156,7 +164,7 @@ sub cgiapp_prerun {
 
 
 sub _untaint_path {
-	my $path = shift;
+	my ($self, $path) = @_;
 
 	return "" if not defined $path;
 	return "" if $path =~ /\.\./;
@@ -196,12 +204,19 @@ sub redirect {
 # method. By default we'll load the template from within our module.
 sub load_tmpl {
 	my $self = shift;
-	my $name = shift;
-	
-	my $template = _get_template($name);
-	croak "Could not load template '$name'" if not $template;
 
-	my $t = HTML::Template->new_scalar_ref(\$template, @_);
+	my $t;
+	if ($self->param("TMPL_PATH")) {
+		$t = $self->SUPER::load_tmpl(@_);
+	} else {
+		my $name = shift;
+	
+		my $template = CGI::FileManager::Templates::_get_template($name);
+		croak "Could not load template '$name'" if not $template;
+
+		$t = HTML::Template->new_scalar_ref(\$template, @_);
+	}
+	
 
 #	my $t = $self->SUPER::load_tmpl(@_, 
 #		      die_on_bad_params => -e ($self->param("ROOT") . "/die_on_bad_param") ? 1 : 0
@@ -287,7 +302,7 @@ sub change_dir {
 	my $self = shift;
 	my $q = $self->query;
 
-	my $workdir = _untaint_path $q->param("workdir");
+	my $workdir = $self->_untaint_path($q->param("workdir"));
 	my $homedir = $self->session->param("homedir");
 
 	my $dir = $q->param("dir");
@@ -368,7 +383,7 @@ sub list_dir {
 
 	my $q = $self->query;
 
-	my $workdir = _untaint_path $q->param("workdir");
+	my $workdir = $self->_untaint_path($q->param("workdir"));
 	my $homedir = $self->session->param("homedir");
 	my $path = File::Spec->catfile($homedir, $workdir);
 
@@ -389,13 +404,13 @@ sub list_dir {
 			my $full = File::Spec->catfile($path, $f);
 			push @files, {
 				filename    => $f,
-				filetype    => _file_type($full),
+				filetype    => $self->_file_type($full),
 				subdir      => -d $full,
 				zipfile     => ($full =~ /\.zip/i ? 1 : 0),
 				filedate    => scalar (localtime((stat($full))[9])),
 				size        => (stat($full))[7],
-				delete_link => $f eq ".." ? "" : _delete_link($full),
-				rename_link => $f eq ".." ? "" : _rename_link($full),
+				delete_link => $f eq ".." ? "" : $self->_delete_link($full),
+				rename_link => $f eq ".." ? "" : $self->_rename_link($full),
 				workdir     => $workdir,
 			};
 		}	
@@ -411,21 +426,21 @@ sub list_dir {
 
 # returns the type of the given file
 sub _file_type {
-	my ($file) = @_;
+	my ($self, $file) = @_;
 	return "dir"  if -d $file;
 	return "file" if -f $file;
 	return "n/a";
 }
 
 sub _delete_link {
-	my ($file) = @_;
+	my ($self, $file) = @_;
 	return "rm=remove_directory;dir="  if -d $file;
 	return "rm=delete_file;filename="  if -f $file;
 	return "";
 }
 
 sub _rename_link {
-	my ($file) = @_;
+	my ($self, $file) = @_;
 	return "rm=rename_form;filename="  if -d $file;
 	return "rm=rename_form;filename="  if -f $file;
 	return "";
@@ -434,7 +449,7 @@ sub _rename_link {
 				
 # Delete a file from the server
 sub delete_file {
-	my $self = shift;
+	my ($self) = @_;
 	my $q = $self->query;
 
 	my $filename = $q->param("filename");
@@ -445,7 +460,7 @@ sub delete_file {
 		return $self->message("Invalid filename. Please contact the system administrator");
 	}
 	my $homedir = $self->session->param("homedir");
-	my $workdir = _untaint_path $q->param("workdir");
+	my $workdir = $self->_untaint_path($q->param("workdir"));
 	
 	$filename = File::Spec->catfile($homedir, $workdir, $filename);
 
@@ -455,7 +470,7 @@ sub delete_file {
 }
 
 sub remove_directory {
-	my $self = shift;
+	my ($self) = @_;
 	my $q = $self->query;
 
 	my $dir = $q->param("dir");
@@ -466,7 +481,7 @@ sub remove_directory {
 		return $self->message("Invalid directory name. Please contact the system administrator");
 	}
 	my $homedir = $self->session->param("homedir");
-	my $workdir = _untaint_path $q->param("workdir");
+	my $workdir = $self->_untaint_path($q->param("workdir"));
 	
 	$dir = File::Spec->catfile($homedir, $workdir, $dir);
 
@@ -489,7 +504,7 @@ sub unzip {
 	}
 
 	my $homedir = $self->session->param("homedir");
-	my $workdir = _untaint_path $q->param("workdir");
+	my $workdir = $self->_untaint_path($q->param("workdir"));
 
 	$filename = File::Spec->catfile($homedir, $workdir, $filename);
 	if (not -e $filename) {
@@ -540,7 +555,7 @@ sub rename {
 	}
 
 	my $homedir = $self->session->param("homedir");
-	my $workdir = _untaint_path $q->param("workdir");
+	my $workdir = $self->_untaint_path($q->param("workdir"));
 
 	$old = File::Spec->catfile($homedir, $workdir, $old);
 	if (not -e $old) {
@@ -574,12 +589,14 @@ sub rename {
 	}
 	return $self->_move($old, $new);
 }
+
+
 sub upload_file {
 	my $self = shift;
 	my $q = $self->query;
 
 	my $homedir = $self->session->param("homedir");
-	my $workdir = _untaint_path $q->param("workdir");
+	my $workdir = $self->_untaint_path($q->param("workdir"));
 
 	my $upload = CGI::Upload->new();
 	my $file_name = $upload->file_name('filename');
@@ -618,7 +635,7 @@ sub create_directory {
 	my $q = $self->query;
 
 	my $homedir = $self->session->param("homedir");
-	my $workdir = _untaint_path $q->param("workdir");
+	my $workdir = $self->_untaint_path($q->param("workdir"));
 	my $dir = $q->param("dir");
 	$dir = _untaint($dir);
 	if (not $dir) {
@@ -760,9 +777,19 @@ A number of users need authentication and full access to one directory tree per
  Move file/directory
  Unzip file (.zip)
 
+=head2 v0.02_02
+
+ Separate CGI::FileManager::Templates
+ add cfm-install.pl install script
+
 
 =head1 TODO
 
+ - install the module as regular CPAN module and add a script that will generate the templates
+   and hard-code their location in the script.
+ 
+ - Replace the Unix::ConfigFile with my own implementation
+
  Test the module on Windows and find out what need to be done to pass the windows
  tests ? Especially look at Unix::ConfigFile
 
@@ -778,8 +805,7 @@ A number of users need authentication and full access to one directory tree per
  Security issues: can I be sure that unzipping a file will open files only under the current directory ?
  What should I do in case a file that comes from an unzip operation already exists ?
 
- ZIP: currently the path to unzip is hard coded. It probably should be replaced by Archive::Zip - I hope
- it can support the operation(s) I need.
+ ZIP: currently the path to unzip is hard coded. It probably should be replaced by Archive::Zip
 
  More fancy things:
  Create file
@@ -787,6 +813,7 @@ A number of users need authentication and full access to one directory tree per
  Unzip file (tar/gz/zip)
  Edit file (simple editor)
 
+ look at CGI::Explorer and check what is the relationsip to it ?
 
 =head1 Author
 
@@ -803,6 +830,7 @@ be notified of progress on your bug as I make changes.
 =head1 Copyright & License
 
 Copyright 2004 Gabor Szabo, All Rights Reserved.
+L<http://www.szabgab.com/>
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -813,329 +841,5 @@ CGI::Upload, WWW::FileManager, CGI::Uploader
 
 =cut
 
-    #<link rel="stylesheet" href="/s/style.css" type="text/css">
-
-sub _get_template {
-	my $name = shift;
-
-	my %tmpl;
-
-my $css = <<CSS;
-<style type="text/css">
-BODY
-{
-    FONT-SIZE: 14px;
-    COLOR: #a5a5a5;
-    FONT-FAMILY: Verdana;
-    BACKGROUND-COLOR: lightblue;
-    TEXT-DECORATION: none
-}
-.error {
-	color: red;
-}
-
-.mybutton {
-    color: #4a4d4a;
-	background-color:#8080FF;
-	font-size:12px;
-	font-weight:bold;
-    /*
-	FONT-FAMILY: Verdana,arial;
-    TEXT-DECORATION: none
-	*/
-}
-.choosebutton {
-    color: #4a4d4a;
-	font-size:12px;
-	font-weight:bold;
-    /*
-	FONT-FAMILY: Verdana,arial;
-    TEXT-DECORATION: none
-	*/
-}
-
-
-A:link
-{
-    FONT-SIZE: 12px;
-    COLOR: #339900;
-    FONT-FAMILY: verdana;
-    TEXT-DECORATION: none
-}
-A:visited
-{
-    FONT-SIZE: 12px;
-    COLOR: #996600;
-    FONT-FAMILY: Verdana;
-    TEXT-DECORATION: none
-}
-A:hover
-{
-    FONT-SIZE: 12px;
-    COLOR: #cc9900;
-    FONT-FAMILY: Verdana;
-    TEXT-DECORATION: none
-}
-A:active
-{
-    FONT-SIZE: 12px;
-    COLOR: #000033;
-    FONT-FAMILY: Verdana;
-    TEXT-DECORATION: none
-}
-
-.files TABLE
-{
-	cell-spacing:   1;
-	cell-padding:   0;
-	border:         1;
-	align:          middle;
-}
-
-.files TH
-{
-	background-color: #8080FF;
-}
-
-TD
-{
-    FONT-SIZE: 13px;
-    COLOR: #4a4d4a;
-    FONT-FAMILY: Verdana,arial;
-    TEXT-DECORATION: none
-}
-
-TH
-{
-	FONT-SIZE: 13px;
-    COLOR: #4a4d4a;
-    FONT-FAMILY: Verdana,arial;
-    TEXT-DECORATION: none
-}
-
-.even TD {
-	FONT-SIZE: 13px;
-    COLOR: #4a4d4a;
-    FONT-FAMILY: Verdana,arial;
-    TEXT-DECORATION: none;
-	background-color: #CCFF99;
-}
-
-.odd TD {
-	FONT-SIZE: 13px;
-    COLOR: #4a4d4a;
-    FONT-FAMILY: Verdana,arial;
-    TEXT-DECORATION: none;
-	background-color: #AAFF99;
-}
-
-
-
-</style>
-CSS
-
-
-$tmpl{message} = <<ENDHTML;
-<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
-<HTML> 
-<HEAD>
-	<TITLE>CGI::FileManager</TITLE>  
-	<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">
-    CSS_STYLE_SHEET
-</HEAD> 
-<body>
-<p>Message</p>
-<TMPL_VAR message>
-</body>
-<HTML>
-ENDHTML
-
-
-$tmpl{login} = <<ENDHTML;
-<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
-<HTML> 
-<HEAD>
-	<TITLE>CGI::FileManager</TITLE>  
-	<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">
-    CSS_STYLE_SHEET
-</HEAD> 
-<body>
-<form method="POST"><br><br><br><br><br><br><br><br>
-<center>
-<TMPL_IF login_failed><div class="error">Login failed</div></TMPL_IF>
-<table bgcolor="#006666" cellspacing="1" cellpadding="0" border="0" align="middle">
-<tr><td bgcolor="#8080FF" colspan=2 align="middle"><B>Login form</B><input type="hidden" name="rm" value="login_process"></td></tr>
-<tr><td bgcolor="#CCFF99">Username:</td> <td><input name="username" value="<TMPL_VAR username>"></td></tr>
-<tr><td bgcolor="#CCFF99">Password:</td> <td> <input name="password" type="password"></td></tr>
-<tr><td bgcolor="#CCFF99" colspan=2 align="middle"><input type="submit" value="Login"></td></tr>
-</table></center>
-</body>
-<HTML>
-ENDHTML
-
-$tmpl{list_dir} = <<ENDHTML;
-<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
-<HTML> 
-<HEAD>
-	<TITLE>CGI::FileManager - Directory Listing</TITLE>  
-	<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">
-    CSS_STYLE_SHEET
-</HEAD> 
-<body><br><br>
-<center>
-
-<TMPL_IF files>
-<div class="files">
-<table>
- <tr>
-   <th>name</th>
-   <th>type</th>
-   <th>date</th>
-   <th>size</th>
-   <th></th>
-   <th></th>
- </tr>
- <TMPL_LOOP files>
-  <tr class="<TMPL_IF NAME="__odd__">odd<TMPL_ELSE>even</TMPL_IF>">
-  	<TMPL_IF subdir>
-	  <td>
-	    <a href="?rm=change_dir;workdir=<TMPL_VAR workdir>;dir=<TMPL_VAR filename>">
-		<TMPL_VAR filename>
-		</a>
-	  </td>
-	<TMPL_ELSE>
-	  <TMPL_IF zipfile>
-	    <td>
-	    <a href="?rm=unzip;workdir=<TMPL_VAR workdir>;filename=<TMPL_VAR filename>">
-		<TMPL_VAR filename>
-		</a>
-		</td>
-	  <TMPL_ELSE>
-	    <td><TMPL_VAR filename></td>
-	  </TMPL_IF>
-	</TMPL_IF>
-	<td><TMPL_VAR filetype></td>
-	<td><TMPL_VAR filedate></td>
-	<td><TMPL_VAR size></td>
-	<td><TMPL_IF delete_link><a href="?workdir=<TMPL_VAR workdir>;<TMPL_VAR delete_link><TMPL_VAR filename>">delete</a></TMPL_IF></td>
-	<td><TMPL_IF rename_link><a href="?workdir=<TMPL_VAR workdir>;<TMPL_VAR rename_link><TMPL_VAR filename>">rename</a></TMPL_IF></td>
-  </tr>
- </TMPL_LOOP>
-</table>
-</div>
-</TMPL_IF>
-
-<br><br>
-<table  cellspacing="0" cellpadding="0" border="0" align="middle">
-<TR><TD align="middle" colspan=2><hr></TD></TR>
-
- <tr><td align="right" valign="top">
-<form method="GET">
-<input type="hidden"  name="rm" value="create_directory">
-<input type="hidden"  name="workdir" value="<TMPL_VAR workdir>">
-<input name="dir" size="15"></TD><TD>
-<input type="submit"  class="mybutton" value="Create Directory">
-</form>
-</TD></TR>
-<TR><TD align="middle" colspan=2><hr></TD></TR>
-<TR><TD colspan=2 align="left">
-
-  <form method="POST" enctype="multipart/form-data">
-  <input type="hidden" name="workdir" value="<TMPL_VAR workdir>">
-  <input type="hidden" name="rm" value="upload_file">
-  <input type="file" size="16" name="filename" class="choosebutton">
-  <input type="submit" class="mybutton" value="Upload">
-  </form>
-
-</TD></TR>
-<TR><TD align="middle" colspan=2><hr></TD></TR>
-<TR><TD align="right"></TD>
-    <TD align="left">
-     <table>
-       <tr>
-	   <td>
-        <form method="GET">
-        <input type="hidden" name="rm" value="list_dir">
-        <input type="hidden" name="workdir" value="<TMPL_VAR workdir>">
-        <input type="submit" class="mybutton" value="Refresh">
-        </form>
-       </td>
-       <td>
-        <form method="GET">
-        <input type="hidden" name="rm" value="logout">
-        <input type="submit" class="mybutton" value="Logout">
-        </form>
-       </td>
-	   </tr>
-	 </table>
-   </TD>
-</TR></table>
-
-<table  cellspacing="0" cellpadding="0" border="0" align="middle">
-
-<TR><TD colspan=2 align="middle">
-You are using CGI::FileManager Version: <TMPL_VAR version>
-<br>
-For help contact <a href="mailto:gabor\@pti.co.il">Gabor Szabo</a>
-</TD></TR></table>
-
-</center>
-</body>
-</html>
-ENDHTML
-
-
-
-
-$tmpl{logout} = <<ENDHTML;
-<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
-<HTML> 
-<HEAD>
-	<TITLE>CGI::FileManager - Good bye</TITLE>  
-	<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">
-    CSS_STYLE_SHEET
-</HEAD> 
-<body>
-<p>
-You were successfully logged out.
-</p>
-<form method="POST">
-<input type="submit" value="Login again">
-</form>
-</body>
-<HTML>
-ENDHTML
-
-$tmpl{rename_form} = <<ENDHTML;
-<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
-<HTML> 
-<HEAD>
-	<TITLE>CGI::FileManager - Rename <TMPL_VAR filename></TITLE>  
-	<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">
-    CSS_STYLE_SHEET
-</HEAD> 
-<body>
-<center>
-<form method="POST" action="?">
-<input type="hidden" name="rm" value="rename">
-<input type="hidden" name="workdir" value="<TMPL_VAR workdir>">
-<input type="hidden" name="filename" value="<TMPL_VAR filename>">
-Rename <TMPL_VAR filename>  to
-<input name="newname" value="<TMPL_VAR newname>">
-<input type="submit" value="Rename">
-</form>
-</center>
-</body>
-<HTML>
-ENDHTML
-
-
-	$tmpl{$name} =~ s/CSS_STYLE_SHEET/$css/;
-
-
-	return $tmpl{$name};
-
-}
-
 1; 
 
